/*
 * SIO Test
 * ------------------------------------------------------------------------
 * Licensed under the BSD license, see LICENSE for details.
 *
 * Copyright (c) 2005 James Forshaw <tyranid@gmail.com>
 * Copyright (c) 2005 Frank Buss <fb@frank-buss.de> (aka Shine)
 *
 * This program reads data from the serial port at 9600 baud and
 * writes it back to it, all from user mode thread with stdout.
 * TODO: stdin doesn't work.
 *
 * $Id$
 */

#include <pspkernel.h>
#include <pspdebug.h>
#include <pspdisplay.h>
#include <pspsdk.h>
#include <stdlib.h>
#include <stdio.h>

/* Define the module info section */
PSP_MODULE_INFO("REMOTE", 0x1000, 1, 1);

/* Define the main thread's attribute value (optional) */
PSP_MAIN_THREAD_ATTR(THREAD_ATTR_USER | THREAD_ATTR_VFPU);

/* Exit callback */
int exit_callback(void)
{
	sceKernelExitGame();
	return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp)
{
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", (void *) exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);

	sceKernelSleepThreadCB();

	return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void)
{
	int thid = 0;

	thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
	if(thid >= 0)
	{
		sceKernelStartThread(thid, 0, 0);
	}

	return thid;
}

int sioOutputHandler(const char *data, int len)
{
	// output on serial port
	int i;
	for (i = 0; i < len; i++) {
		char c = data[i];
		
		// remove this line, if you want binary mode streams
		// or if you have more clever programs than Hyperterminal
		if (c == '\n') pspDebugSioPutchar('\r');

		pspDebugSioPutchar(c);
	}
	return i;
}

int sioInputHandler(char *data, int len)
{
	// read from serial port
	int i;
	for (i = 0; i < len; i++) {
		int byte = pspDebugSioGetchar();
		if (byte < 0) break;
		data[i] = byte;
	}
	return i;
}

__attribute__((constructor)) void stdoutInit() 
{ 
	pspKernelSetKernelPC();
	
	// power up serial port
	pspDebugSioInit();
	
	// set baud rate
	pspDebugSioSetBaud(9600);
	
	// install handlers
	pspDebugInstallStdinHandler(sioInputHandler); 
	pspDebugInstallStdoutHandler(sioOutputHandler); 
	pspDebugInstallStderrHandler(sioOutputHandler); 
} 

int main(int argc, char** argv)
{
	SetupCallbacks();
	
	pspDebugScreenInit();

	pspDebugScreenPrintf("SIO Test\n");
	printf("\r\nPSP serial echo test. Type some text for echo.\r\n\r\n");

	char data[256];
	int ping = 0;
	int myStdin = sceIoOpen("tty0:", PSP_O_RDONLY, 0);
	while(1) {
		int count = sceIoRead(myStdin, data, 256);
		if (count > 0) {
			// terminal programs like Hyperterminal sends just a \r for return key, convert it
			int i = 0;
			for (i = 0; i < count; i++) if (data[i] == '\r') data[i] = '\n';
			pspDebugScreenPrintData(data, count);

			// echo data to serial port
			fwrite(data, 1, count, stdout);
		}
		if (ping++ == 60) {
			printf(".");
			ping = 0;
		}
		fflush(stdout);
		sceDisplayWaitVblankStart();
	}
	sceIoClose(myStdin);

	return 0;
}
